

#include "mcc_generated_files/mcc.h"

unsigned char writeSPIByte(unsigned char data);
void nrf_init();
void reset_buffer();
void spi_init();

#define SPI_RX_IN_PROGRESS 0x0
// Defines
#define LED_PIN1 LATAbits.LATA1 //led test
#define LED_PIN2 LATAbits.LATA2

#define SW PORTBbits.RB7

#define CE_Pin LATCbits.LATC5 //set port as output
#define CSN_Pin LATCbits.LATC6 //set port as output
#define SCK_Pin LATCbits.LATC3 //set port as output
#define MISO_Pin PORTCbits.RC4 //set port as input
#define MOSI_Pin LATCbits.LATC2 //set port as output
#define IRQ_Pin PORTCbits.RC1


#define HIGH 1
#define LOW 0
#define NRF24
#define nrf24_ADDR_LEN 5
#define nrf24_CONFIG ((1<<EN_CRC)|(0<<CRCO))
#define NRF24_TRANSMISSON_OK 0
#define NRF24_MESSAGE_LOST 1

uint8_t PTX;
uint8_t payload;
uint8_t channel;
uint8_t data_array[4];

void main(void)
{
    SYSTEM_Initialize();
    spi_init();
    nrf_init();
    LED_PIN1 = 0;
    __delay_ms(100);

    while (1) 
    {
            if (IRQ_Pin == 0) 
            {
                LED_PIN1 = 1; __delay_ms(1000);
                reset_buffer();
            }   
            LED_PIN1 = 0; __delay_ms(200);
    }
}

unsigned char writeSPIByte(unsigned char data)
{
    uint8_t dummyRead = SSP1BUF;    // Clears buffer full flag

    // Clear the Write Collision flag, to allow writing
    SSP1CON1bits.WCOL = 0;

    SSP1BUF = data;

    while(SSP1STATbits.BF == SPI_RX_IN_PROGRESS)
    {
    }

    return (SSP1BUF);
}

void nrf_init()
{
    unsigned char j;

    CE_Pin = LOW;
    __delay_ms( 1 );
    CSN_Pin = HIGH;

    __delay_ms(100);

    //PRX, CRC enabled
    //XXX unmasked interrupts
    CSN_Pin = LOW;
    writeSPIByte(0x20);
    writeSPIByte(0x09);
    CSN_Pin = HIGH;

    //disable auto-ack for all channels
    CSN_Pin = LOW;
    writeSPIByte(0x21);
    writeSPIByte(0x00);
    CSN_Pin = HIGH;

    //address width = 5 bytes
    CSN_Pin = LOW;
    writeSPIByte(0x23);
    writeSPIByte(0x03);
    CSN_Pin = HIGH;

    //data rate = 1MB
    CSN_Pin = LOW;
    writeSPIByte(0x26);
    writeSPIByte(0x06);
    CSN_Pin = HIGH;

    //4 byte payload
    CSN_Pin = LOW;
    writeSPIByte(0x31);
    writeSPIByte(0x04);
    CSN_Pin = HIGH;

    //set channel 2
    //XXX changed to channel 5
    CSN_Pin = LOW;
    writeSPIByte(0x25);
    writeSPIByte(0x05);
    CSN_Pin = HIGH;

    //set pipe0 address
    //XXX fixed command, changed address
    const char *addr = "serv1";
    CSN_Pin = LOW;
    writeSPIByte(0x2A);
    for (j = 0; j < 5; j++) {
        writeSPIByte( addr[j] );
    }
    CSN_Pin = HIGH;

    //PWR_UP = 1
    /// XXX unmasked interrupts
    CSN_Pin = LOW;
    writeSPIByte(0x20);
    writeSPIByte(0x0B);
    CSN_Pin = HIGH;
    __delay_ms( 1 );

    CE_Pin = HIGH;
}

void reset_buffer()
{
    unsigned char j;
    unsigned char buffer[4];

    //Read RX payload
    CSN_Pin = LOW;
    writeSPIByte(0x61);
    for (j = 0; j < 4; j++) {
        buffer[j] = writeSPIByte(0);
    }
    CSN_Pin = HIGH;
    __delay_ms(1);

#ifdef CRUFT
    //XXX not needed after read
    //Flush RX FIFO
    CSN_Pin = LOW;
    writeSPIByte(0xE2);
    CSN_Pin = HIGH;
    __delay_ms( 1 );
#endif
    
    //reset int
    CSN_Pin = LOW;
    writeSPIByte(0x27);
    writeSPIByte(0x40);
    CSN_Pin = HIGH;
}

void spi_init()
{
    TRISC6 = 0; 
    TRISC2 = 0; 
    TRISC3 = 0; 
    TRISC4 = 1; 
    TRISC5 = 0; 
    TRISC1 = 1; 
}
